@@ -13,7 +13,7 @@ from contextlib import nullcontext
 from pathlib import Path
 
 import hy
-from hy._compat import PY3_9
+from hy.compat import PY3_9
 from hy.compiler import hy_compile, hy_eval
 from hy.errors import HyLanguageError, filtered_hy_exceptions, hy_exc_handler
 from hy.importer import runhy
@@ -262,10 +262,7 @@ def cmdline_handler(argv):
         return 0
     elif action == "run_script_stdin":
         sys.argv = argv
-        if repl:
-            source = sys.stdin
-            filename = 'stdin'
-        else:
+        if not repl:
             return run_command(sys.stdin.read(), filename="<stdin>")
     elif action == "run_script_file":
         sys.argv = argv
