@@ -528,6 +528,7 @@ def compile_assign(
             ann_result = compiler.compile(ann)
             result = ann_result + result
 
+        target = dict(target = st_name)
         if is_assignment_expr:
             node = asty.NamedExpr
         elif ann is not None:
@@ -539,12 +540,12 @@ def compile_assign(
             )
         else:
             node = asty.Assign
+            target = dict(targets = [st_name])
 
         result += node(
             name if hasattr(name, "start_line") else result,
             value=result.force_expr if not annotate_only else None,
-            target=st_name,
-            targets=[st_name],
+            **target
         )
 
     return result
@@ -1115,11 +1116,12 @@ def compile_with_expression(compiler, expr, root, args, body):
         expr, targets=[name], value=asty.Constant(expr, value=None)
     )
 
+    [args] = args
     ret = Result(stmts=[initial_assign])
     items = []
     was_async = None
     cbody = None
-    for i, (is_async, variable, ctx) in enumerate(args[0]):
+    for i, (is_async, variable, ctx) in enumerate(args):
         is_async = bool(is_async)
         if was_async is None:
             was_async = is_async
@@ -1127,17 +1129,33 @@ def compile_with_expression(compiler, expr, root, args, body):
             # We're compiling a `with` that mixes synchronous and
             # asynchronous context managers. Python doesn't support
             # this directly, so start a new `with` inside the body.
-            cbody = compile_with_expression(compiler, expr, root, [args[0][i:]], body)
+            cbody = compile_with_expression(compiler, expr, root, [args[i:]], body)
             break
-        ctx = compiler.compile(ctx)
-        ret += ctx
+        if not isinstance(ctx, Result):
+            # In a non-recursive call, `ctx` has not yet been compiled,
+            # and thus is not yet a `Result`.
+            ctx = compiler.compile(ctx)
+            if i == 0:
+                ret += ctx
+            elif ctx.stmts:
+                # We need to include some statements as part of this
+                # context manager, but this `with` already has at
+                # least one prior context manager. So, put our
+                # statements in the body and then start a new `with`.
+                cbody = ctx + compile_with_expression(
+                    compiler,
+                    expr,
+                    root,
+                    [((is_async, variable, ctx), *args[i + 1:])],
+                    body)
+                break
         variable = (
             None
             if variable == Symbol("_")
             else compiler._storeize(variable, compiler.compile(variable))
         )
         items.append(
-            asty.withitem(expr, context_expr=ctx.force_expr, optional_vars=variable)
+            ast.withitem(context_expr=ctx.force_expr, optional_vars=variable)
         )
 
     if not cbody:
@@ -1211,7 +1229,6 @@ def compile_match_expression(compiler, expr, root, subject, clauses):
                     name=fname,
                     args=ast.arguments(
                         args=[],
-                        varargs=None,
                         kwarg=None,
                         posonlyargs=[],
                         kwonlyargs=[],
@@ -1356,9 +1373,7 @@ def compile_raise_expression(compiler, expr, root, exc, cause):
         ret += cause
         cause = cause.force_expr
 
-    return ret + asty.Raise(
-        expr, type=ret.expr, exc=exc, inst=None, tback=None, cause=cause
-    )
+    return ret + asty.Raise(expr, exc=exc, cause=cause)
 
 
 @pattern_macro(
@@ -1780,8 +1795,6 @@ def compile_class_expression(compiler, expr, root, decorators, tp, name, rest):
         decorator_list=decorators,
         name=name,
         keywords=keywords,
-        starargs=None,
-        kwargs=None,
         bases=bases_expr,
         body=bodyr.stmts or [asty.Pass(expr)],
         **digest_type_params(compiler, tp)
@@ -1965,35 +1978,34 @@ def compile_import(compiler, expr, root, entries):
         elif assignments == "EXPORTS":
             compiler.scope.define(prefix)
             node = asty.Import
-            names = [
-                asty.alias(
-                    module,
-                    name=module_name,
-                    asname=prefix if prefix != module_name else None,
-                )
-            ]
+            names = [asty.alias(
+                module,
+                name = module_name,
+                asname = prefix if prefix != module_name else None)]
         else:
             node = asty.ImportFrom
             names = []
             for k, v in assignments:
                 compiler.scope.define(mangle(v))
-                names.append(
-                    asty.alias(
-                        module, name=mangle(k), asname=None if v == k else mangle(v)
-                    )
-                )
+                names.append(asty.alias(
+                    module,
+                    name = mangle(k),
+                    asname = None if v == k else mangle(v)))
         ret += node(
             expr,
-            module=module_name if module_name and module_name.strip(".") else None,
-            names=names,
-            level=(
-                len(module[0])
-                if isinstance(module, Expression) and module[1][0] == Symbol("None")
-                else len(module)
-                if isinstance(module, Symbol) and not module.strip(".")
-                else 0
-            ),
-        )
+            names = names,
+            **({} if node is asty.Import else dict(
+                module = module_name
+                    if module_name and module_name.strip(".")
+                    else None,
+                level =
+                    len(module[0])
+                    if isinstance(module, Expression)
+                        and module[1][0] == Symbol("None")
+                    else len(module)
+                    if isinstance(module, Symbol)
+                        and not module.strip(".")
+                    else 0)))
 
     return ret
 
