@@ -6,9 +6,9 @@ import importlib.util
 from contextlib import contextmanager
 
 from hy import _initialize_env_var
-from hy._compat import PYPY
+from hy.compat import PYPY, PY3_13
 
-_hy_filter_internal_errors = _initialize_env_var("HY_FILTER_INTERNAL_ERRORS", True)
+_hy_show_internal_errors = _initialize_env_var("HY_SHOW_INTERNAL_ERRORS", False)
 
 
 class HyError(Exception):
@@ -113,7 +113,7 @@ class HyLanguageError(HyError):
         )
 
         arrow_idx, _ = next(
-            ((i, x) for i, x in enumerate(output) if x.strip() == "^"), (None, None)
+            ((i, x) for i, x in enumerate(output) if set(x.strip()) == {"^"}), (None, None)
         )
         if arrow_idx:
             msg_idx = arrow_idx + 1
@@ -125,8 +125,10 @@ class HyLanguageError(HyError):
         # Get rid of erroneous error-type label.
         output[msg_idx] = re.sub("^SyntaxError: ", "", output[msg_idx])
 
-        # Extend the text arrow, when given enough source info.
-        if arrow_idx and self.arrow_offset:
+        # Extend the text arrow, when given enough source info. We
+        # don't do this on newer Pythons because they make their own
+        # underlines.
+        if arrow_idx and self.arrow_offset and not PY3_13:
             output[arrow_idx] = "{}{}^\n".format(
                 output[arrow_idx].rstrip("\n"), "-" * (self.arrow_offset - 1)
             )
@@ -146,10 +148,6 @@ class HyTypeError(HyLanguageError, TypeError):
     """TypeError occurring during the normal use of Hy."""
 
 
-class HyNameError(HyLanguageError, NameError):
-    """NameError occurring during the normal use of Hy."""
-
-
 class HyRequireError(HyLanguageError):
     """Errors arising during the use of `require`
 
@@ -175,12 +173,6 @@ class HyEvalError(HyLanguageError):
     """
 
 
-class HyIOError(HyInternalError, IOError):
-    """Subclass used to distinguish between IOErrors raised by Hy itself as
-    opposed to Hy programs.
-    """
-
-
 class HySyntaxError(HyLanguageError, SyntaxError):
     """Error during the Lexing of a Hython expression."""
 
@@ -231,7 +223,7 @@ _tb_hidden_modules = {
             "hy.repl",
             "hy.reader.parser",
             "hy.importer",
-            "hy._compat",
+            "hy.compat",
             "hy.macros",
             "hy.models",
             "hy.core.result_macros",
@@ -280,11 +272,11 @@ def hy_exc_filter(exc_type, exc_value, exc_traceback):
 
 def hy_exc_handler(exc_type, exc_value, exc_traceback):
     """A `sys.excepthook` handler that uses `hy_exc_filter` to
-    remove internal Hy frames from a traceback print-out.
+    remove internal Hy frames from a traceback print-out, so long
+    as `_hy_show_internal_errors` is false.
     """
-    if os.environ.get("HY_DEBUG", False):
+    if _hy_show_internal_errors:
         return sys.__excepthook__(exc_type, exc_value, exc_traceback)
-
     try:
         output = hy_exc_filter(exc_type, exc_value, exc_traceback)
         sys.stderr.write(output)
@@ -299,14 +291,10 @@ def filtered_hy_exceptions():
     from tracebacks.
 
     Filtering can be controlled by the variable
-    `hy.errors._hy_filter_internal_errors` and environment variable
-    `HY_FILTER_INTERNAL_ERRORS`.
+    `hy.errors._hy_show_internal_errors` and environment variable
+    `HY_SHOW_INTERNAL_ERRORS`.
     """
-    global _hy_filter_internal_errors
-    if _hy_filter_internal_errors:
-        current_hook = sys.excepthook
-        sys.excepthook = hy_exc_handler
-        yield
-        sys.excepthook = current_hook
-    else:
-        yield
+    current_hook = sys.excepthook
+    sys.excepthook = hy_exc_handler
+    yield
+    sys.excepthook = current_hook
